#ifndef _QLCNIC_COMPAT_H
#define _QLCNIC_COMPAT_H

#ifndef DEFINE_PCI_DEVICE_TABLE
#define DEFINE_PCI_DEVICE_TABLE(_table) \
         struct pci_device_id _table[] __devinitdata
#endif

#ifndef phys_addr_t
typedef unsigned long long phys_addr_t;
#endif

#ifndef resource_size_t
typedef phys_addr_t resource_size_t;
#endif

#ifndef MODULE_FIRMWARE
#define MODULE_FIRMWARE(s)
#endif

#ifndef skb_copy_from_linear_data
static inline void skb_copy_from_linear_data(const struct sk_buff *skb,
					     void *to,
					     const unsigned int len)
{
	memcpy(to, skb->data, len);
}
#endif

#ifndef skb_copy_from_linear_data_offset
static inline void skb_copy_from_linear_data_offset(const struct sk_buff *skb,
						    const int offset, void *to,
						    const unsigned int len)
{
	memcpy(to, skb->data + offset, len);
}
#endif

#ifndef round_jiffies_relative
#define round_jiffies_relative(JIFFIES)	JIFFIES
#endif

#ifndef PCI_MSIX_FLAGS_ENABLE
#define PCI_MSIX_FLAGS_ENABLE	(1 << 15)
#endif

#ifndef CHECKSUM_PARTIAL
#define CHECKSUM_PARTIAL	CHECKSUM_HW
#endif

#ifndef NETIF_F_TSO6
#define NETIF_F_TSO6	0
#endif

#ifndef NETIF_F_IPV6_CSUM
#define NETIF_F_IPV6_CSUM	0
#endif

#ifndef IRQF_SAMPLE_RANDOM
#define IRQF_SAMPLE_RANDOM	SA_SAMPLE_RANDOM
#define IRQF_SHARED		SA_SHIRQ
typedef irqreturn_t (*irq_handler_t)(int , void *, struct pt_regs *);
#endif

#ifndef FIELD_SIZEOF
#define FIELD_SIZEOF(t, f) (sizeof(((t*)0)->f))
#endif

#ifndef DMA_BIT_MASK
#define DMA_BIT_MASK(n) (((n) == 64) ? ~0ULL : ((1ULL<<(n))-1))
#endif

#ifndef skb_transport_offset
#define skb_network_offset(skb)         ((skb)->nh.raw - (skb)->data)
#define skb_transport_offset(skb)       ((skb)->h.raw - (skb)->data)
#endif

#ifndef tcp_hdrlen
#define tcp_hdrlen(skb)                 (((skb)->h.th)->doff << 2)
#endif

#ifndef ip_hdr
#define ip_hdr(skb)			((skb)->nh.iph)
#endif

#ifndef ipv6_hdr
#define ipv6_hdr(skb)			((skb)->nh.ipv6h)
#endif

#ifndef PORT_OTHER
#define PORT_OTHER	0xff
#endif

#ifndef netdev_tx_t
#define netdev_tx_t	int
#endif

#ifndef pci_ioremap_bar
#define pci_ioremap_bar(PDEV, BAR) \
	ioremap_nocache(pci_resource_start(PDEV, BAR), \
			pci_resource_len(PDEV, BAR));
#endif

#ifndef list_splice_tail_init
#define list_splice_tail_init list_splice_init
#endif

#ifndef work_func_t
typedef void (*work_func_t)(void *);
#endif

#ifndef vlan_dev_real_dev
#define vlan_dev_real_dev(dev) (VLAN_DEV_INFO(dev)->real_dev)
#endif

#ifndef NETIF_F_LRO
#define	NETIF_F_LRO		32768		/* large receive offload */
#endif

#ifndef to_net_dev
#define to_net_dev(class) container_of(class, struct net_device, class_dev)
#endif

#ifndef DEFINE_PCI_DEVICE_TABLE
#define DEFINE_PCI_DEVICE_TABLE(_table) \
         struct pci_device_id _table[] __devinitdata
#endif

#ifndef netdev_mc_count
#define netdev_mc_count(dev) ((dev)->mc_count)
#endif

#ifndef netdev_mc_empty
#define netdev_mc_empty(dev) (netdev_mc_count(dev) == 0)
#endif

#ifndef netdev_for_each_mc_addr
#define netdev_for_each_mc_addr(mclist, dev) \
	for (mclist = dev->mc_list; mclist; mclist = mclist->next)
#endif

#ifndef fls_long
static inline unsigned fls_long(unsigned long l)
{
	if (sizeof(l) == 4)
		return fls(l);
	return fls64(l);
}
#endif

#ifndef memory_read_from_buffer
static inline ssize_t memory_read_from_buffer(void *to, size_t count, loff_t *ppos,
	const void *from, size_t available)
{
	loff_t pos = *ppos;

	if (pos < 0)
		return -EINVAL;
	if (pos >= available)
		return 0;
	if (count > available - pos)
		count = available - pos;
	memcpy(to, from + pos, count);
	*ppos = pos + count;

	return count;
}
#endif

#ifndef bool
typedef _Bool                   bool;
#endif


#ifndef is_power_of_2
static inline __attribute__((const))
bool is_power_of_2(unsigned long n)
{
	return (n != 0 && ((n & (n - 1)) == 0));
}
#endif

#ifndef __rounddown_pow_of_two
static inline __attribute__((const))
unsigned long rounddown_pow_of_two(unsigned long n)
{
	return 1UL << (fls_long(n) - 1);
}
#endif

#endif
